<?php

namespace Alive\Dm;

/**
 * Discount search interface shortcode handler.
 */
class Discount_Search extends Short_Code {
    const VERSION = 'Version';
    /**
     * @var array Default attributes for the shortcode.
     */
    private $default_shortcode_attributes = array(
        'card_types' => array('isic', 'itic', 'iytc', 'alive', 'eyca', 'isic_scholar'),
        'list_header' => '',
        'card_types_brands' => '',
    );

    private $dm_client;

    private $context = null;

    /**
     * @var array Active provider instance holder.
     */
    private $active_context_provider = null;

    /**
     * Initialize the handler.
     */
    public function init() {
        parent::init();
        add_filter('query_vars', array($this, 'query_vars_filter'));
        add_filter('rewrite_rules_array', array($this, 'register_url_rewrite_rules'), 100);
        add_filter('wp_title', array($this, 'custom_title'), 100);
        add_action('wp_head', array($this, 'generate_social_metatags'));
        add_action('wp_head', array($this, 'generate_metatags'));
        add_action('wp_loaded', array($this, 'flush_rewrite_rules'));
        if (function_exists('rel_canonical')) {
            remove_action('wp_head', 'rel_canonical');
        }
        $this->context = $this->get_context();
        $this->checkCanonicalUrl();
    }

    /**
     * Set SEO title for the discount detail.
     *
     * The title should consist of:
     * * SEO Title of the benefit
     * * Page title
     * * Blog title
     *
     * XXX: Unfortunatelly the templte used in production always prepends the
     * blog title to the beginning. see presscore_blog_title() in dt-the7/inc/helpers.php.
     *
     * @param string $title Current page title.
     * @return string Updated title.
     */
    public function custom_title($title) {
        $provider = $this->get_active_context_provider();
        if ($provider != null) {
            return Localizer::get_provider_value($provider, 'seoTitle');
        }
        return $title;
    }


    public function generate_metatags() {
        $provider = $this->get_active_context_provider();
        if ($provider == null) {
            return;
        }
        echo "<meta name='description' content='" . Localizer::get_provider_value($provider, 'metaDescription') . "' />";
    }

    /**
     * Register rewrite rules necessary for this plugin.
     *
     * XXX: I don't think that manipulating the rewrites directly is optimal, but it works for now.
     *
     * @param array $rules Currently ser WP rewrite rules.
     * @return array Updated array of rewrite rules.
     */
    public function register_url_rewrite_rules($rules) {
        return $this->getProviderDetailUrlRewriteRules() + $rules;
    }

    /**
     * @deprecated this really needs to be handled differently. (e.g. when configuration is updated).
     * Thi is potential cause of performance problems as it flushes cache with every request to this page.
     */
    public function flush_rewrite_rules() {
        // FIXME: Might cause some performance issues.
        global $wp_rewrite;
        $wp_rewrite->flush_rules();
    }

    /**
     * Get provider canoncal URL.
     * @param array $provider Provider object.
     * @param string $pagename Name of the page where plugin is included.
     * @return string Canonical provider URL.
     */
    private function get_provider_url($provider, $pagename = null) {
        $uri          = $_SERVER['REQUEST_URI'];
        $parsed_uri   = parse_url('http://' . $_SERVER['HTTP_HOST'] . $uri);
        $provider_url = empty($pagename) ? get_permalink() : get_site_url() . '/' . $pagename . '/';
        $provider_url .= Localizer::get_provider_value($provider, 'slug') . '/' . $provider->id . '/';
        $query = array();
        if (in_array('query', $parsed_uri)) {
            parse_str($parsed_uri['query'], $query);
            foreach ($query as $k => $v) {
                if (substr($k, 0, 2) == 'dm' && $k != 'dmCardType') {
                    unset($query[$k]);
                }
            }
        }

        if (!empty($query)) {
            $provider_url .= '?' . http_build_query($query);
        }
        return $provider_url;
    }

    /**
     * Generates metatags from facebook opengrap api.
     *
     * og:title, og:description, og:image, og:url, og:type
     * FIXME: To přímé vkládání na výstup mi přijde divoké, raději bych to viděl v proměnné (nebo v templatu, který ale bohužel nepoužíváme).
     */
    public function generate_social_metatags() {
        // show actual plugin version
        if( ! function_exists( 'get_plugin_data' ) ) {
            require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
        }
        $data = get_plugin_data(ABSPATH . "/wp-content/plugins/alive-dm-plugin/alive-dm-plugin.php", false, false );
        if (isset($data[self::VERSION])) {
            echo "<script>console.log('DM plugin Version: {$data[self::VERSION]}')</script>";
        }

        $provider = $this->get_active_context_provider();
        if ($provider == null) {
            if (function_exists('rel_canonical')) {
                rel_canonical();
            }

            return;
        }
        ?>
                                            <meta property="og:title" content="<?php echo get_bloginfo( 'name' ); ?> - <?php echo esc_attr(Localizer::get_provider_value($provider, 'name'))?>" />
        <?php if (!empty($provider->discounts)): ?>
             <meta property="og:description" content="<?php
            $discounts = '';
            foreach ($provider->discounts as $discount) {
                if (!empty($discounts)) {
                    $discounts .= ' | ';
                }
                $discounts .= Localizer::get_discount_value($discount, 'name');
            }
            echo esc_attr($discounts);
            ?>" />
        <?php endif; ?>
                                            <meta property="og:image" content="<?php echo esc_attr($provider->imageUrl);?>" />
                                            <meta property="og:url" content="<?php echo esc_attr($this->get_provider_url($provider))?>" />
                                            <meta property="og:type" content="website" />
                                            <link rel='canonical' href='<?php echo esc_attr($this->get_provider_url($provider))?>' />
        <?php
    }

    /**
     * Query vars filter for page with included shortcode. In order to get query parameters belonging to this plugin.
     * @param array $vars Array of allowed variable names (already present in application).
     * @return array Array of allowed variable names (including names added for the shortcode).
     */
    function query_vars_filter($vars) {
        $vars[] = "dmDisplay";
        $vars[] = "dmProviderId";
        $vars[] = "dmText";
        $vars[] = "dmCategoryId";
        $vars[] = "dmCardType";
        $vars[] = "dmCrawlPage";
        return $vars;
    }

    /**
     * Get array of variables representitng curent context of the application.
     * @return array Array with variables representing the context. Can be empty, never null.
     */
    private function get_context() {
        $context               = array();
        $context['display']    = get_query_var('dmDisplay');
        $context['providerId'] = get_query_var('dmProviderId');
        $context['text']       = get_query_var('dmText');
        $context['categoryId'] = get_query_var('dmCategoryId');
        $context['cardTypeId'] = get_query_var('dmCardType');
        $context['crawlPage']  = get_query_var('dmCrawlPage');
        return $context;
    }

    /**
     * Get shortcode name.
     * @return string Name of the shortcode. Never null.
     */
    protected function short_code() {
        return 'dm_full_discounts';
    }

    /**
     * Compile categories that will be included to the JS.
     * @param array $categories Array of categories.
     * @return array Array of mapped categories.
     */
    private function compileCategories($categories) {
        $categoryMap = array();
        foreach ($categories as $category) {
            $categoryMap[$category->id] = array(
                'imageUrl' => $this->getCategoryImageUrl($category, 'NORMAL'),
                'icon'     => isset($category->icon) ? $category->icon : 'other'
            );
            foreach ($category->childCategories as $category) {
                $categoryMap[$category->id] = array(
                    'imageUrl' => $this->getCategoryImageUrl($category, 'NORMAL'),
                    'icon'     => isset($category->icon) ? $category->icon : 'other'
                );
            }
        }
        return $categoryMap;
    }

    private function getCategoryImageUrl($category, $type = null) {
        if ($type == null) {
            return isset($category->imageUrl) ? $category->imageUrl : null;
        }
        if (isset($category->image)) {
            foreach ($category->image as $image) {
                if ($image->type == $type) {
                    return $image->url;
                }
            }
        }
        return isset($category->imageUrl) ? $category->imageUrl : null;
    }

    /**
     * (non-PHPdoc)
     * @see \Alive\Dm\Short_Code::load_scripts()
     */
    public function load_scripts() {
        // Include css files.namespace Alive\Sport
        wp_enqueue_style("pretty-photo",
            plugins_url("resources/prettyPhoto/css/prettyPhoto.css", dirname(__FILE__))
        );
        wp_enqueue_style("discount-search",
            plugins_url("resources/alive-dm/css/discount_search.css", dirname(__FILE__))
        );
        wp_enqueue_style("font-awesome",
            plugins_url("resources/font-awesome-4.1.0/css/font-awesome.min.css", dirname(__FILE__))
        );
        wp_enqueue_style("maginfic-popup",
            plugins_url("resources/magnific-popup/magnific-popup.css", dirname(__FILE__))
        );
        // Include javascripts.
        wp_enqueue_script('jquery');
        if (Config::map_provider() === 'google') {
            wp_enqueue_script('alive-dm-plugin-gmap',
                '//maps.googleapis.com/maps/api/js?key=' . Config::google_api_key()
            );
        }
        wp_enqueue_script('alive-dm-plugin-street',
            '//unpkg.com/leaflet@1.6.0/dist/leaflet.js'
        );
        wp_enqueue_style('alive-dm-plugin-gmap',
            '//unpkg.com/leaflet@1.6.0/dist/leaflet.css'
        );
        wp_enqueue_script('alive-dm-plugin-pixastic',
            plugins_url('resources/alive-dm/js/pixastic.custom.js', dirname(__FILE__))
        );
        wp_enqueue_script('alive-dm-plugin-jqueryextensions',
            plugins_url('resources/alive-dm/js/jquery_extensions.js', dirname(__FILE__)),
            array('alive-dm-plugin-pixastic')
        );
        wp_enqueue_script('alive-dm-plugin-social',
            plugins_url('resources/alive-dm/js/social.js', dirname(__FILE__))
        );
        wp_enqueue_script('alive-dm-plugin-prettyphoto',
            plugins_url('resources/prettyPhoto/js/jquery.prettyPhoto.js', dirname(__FILE__))
        );
        wp_enqueue_script('alive-dm-plugin-prettyphotoloader',
            plugins_url('resources/alive-dm/js/prettyPhotoLoader.js', dirname(__FILE__))
        );
        wp_enqueue_script('alive-dm-plugin-magnificpopup',
            plugins_url('resources/magnific-popup/jquery.magnific-popup.js', dirname(__FILE__))
        );
        wp_enqueue_script('jquery-ui-autocomplete');
        wp_enqueue_script("alive-dm-plugin-discount-search",
            plugins_url("resources/alive-dm/js/discount_search.js", dirname(__FILE__)),
            array('alive-dm-plugin-pixastic', 'alive-dm-plugin-social', 'alive-dm-plugin-jqueryextensions')
        );
        // Localize scripts
        wp_localize_script("alive-dm-plugin-discount-search", "discount_search",
            array(
                // XXX Předávání konfigurace přes lokalizaci je opravdu špatně (už to přineslo i problémy).
                // Správně by se konfigurace měla předat až při inicializaci JS toho vyhledávače.
                'provider_logo_url'                          => home_url('/'),
                'plugins_url'                                => plugins_url('alive-dm-plugin'),
                'categories_sprites_url'                     => plugins_url('resources/alive-dm/img/categories.png', dirname(__FILE__)),
                'white_plus_overlay_url'                     => plugins_url('resources/alive-dm/img/white-plus.png', dirname(__FILE__)),
                'white_play_overlay_url'                     => plugins_url('resources/alive-dm/img/play.png', dirname(__FILE__)),
                'empty_image_url'                            => plugins_url('resources/alive-dm/img/empty-image.png', dirname(__FILE__)),
                'default_provider_img'                       => plugins_url('resources/alive-dm/img/default.jpg', dirname(__FILE__)),
                'map_provider'                               => Config::map_provider(),
                'initial_lat'                                => Config::lat(),
                'initial_lon'                                => Config::lon(),
                'initial_zoom'                               => Config::zoom(),
                'currency'                                   => Config::currency(),
                'currency_pos'                               => Config::currency_pos(),
                'isic_url'                                   => Config::isic_url(),
                'isic_scholar_url'                           => Config::isic_scholar_url(),
                'itic_url'                                   => Config::itic_url(),
                'iytc_url'                                   => Config::iytc_url(),
                'alive_url'                                  => Config::alive_url(),
                'eyca_url'                                   => Config::eyca_url(),
                'number_format'                              => number_format_i18n(1000, 1),
                'is_user_logged_in'                          => is_user_logged_in() ? 'true' : 'false',
                'twitter_account'                            => Config::twitter(),
                'blog_name'                                  => get_bloginfo('name'),
                // ad XXX Odtud dále už jsou to pouze překladové hlášky, což je v pořádku.
                'list_images_viewDiscount'                   => __dm('View discount'),
                'list_images_moreBranches'                   => __dm('More branches'),
                'detail_offeredDiscounts'                    => __dm('Offered discounts'),
                'discount_restriction_notForDiscountedItems' => __dm('Does not apply to discounted items'),
                'discount_useOnline'                         => __dm('USE ONLINE'),
                'discount_discount'                          => __dm('discount'),
                'discount_type_FREE_1PLUS1'                  => __dm('1 + 1 free'),
                'discount_type_FREE_2PLUS1'                  => __dm('2 + 1 free'),
                'discount_type_FREE_3PLUS1'                  => __dm('3 + 1 free'),
                'discount_type_FREE_4PLUS1'                  => __dm('4 + 1 free'),
                'discount_type_FREE_SHIPPING'                => __dm('Free shipping'),
                'discount_type_FREE_GIFT'                    => __dm('Free gifr'),
                'discount_type_SPECIAL'                      => __dm('Special offer'),
                'detail_website'                             => __dm('Website'),
                'detail_map_header'                          => __dm('Branch, where you can apply the discount'),
                'detail_map_header_s'                        => __dm('Branches, where you can apply the discount'),
                'detail_email'                               => __dm('E-mail'),
                'detail_phone'                               => __dm('Phone'),
                'detail_photos_videos'                       => __dm('Photos and Videos'),
                'detail_tags'                                => __dm('Tags'),
                'detail_similarDiscounts'                    => __dm('Similar discounts'),
                'detail_restrictions_onlyAvailable'          => __dm("The discount is only available"),
                'detail_restrictions_on'                     => __dm('on'),
                'detail_restrictions_at'                     => __dm('at'),
                'detail_restrictions_minimalCost'            => __dm('The minimal order cost is'),
                'detail_restrictions_interval_one'           => __dm('once'),
                'detail_restrictions_interval_many'          => __dm('%limit% times'),
                'detail_restrictions_interval_day_one'       => __dm('The discount can be used only %limit% a day.'),
                'detail_restrictions_interval_day_few'       => __dm('The discount can be used only %limit% in a %few% days.'),
                'detail_restrictions_interval_day_many'      => __dm('The discount can be used only %limit% in a %many% days.'),
                'detail_restrictions_interval_week_one'      => __dm('The discount can be used only %limit% a week.'),
                'detail_restrictions_interval_week_few'      => __dm('The discount can be used only %limit% in a %few% weeks.'),
                'detail_restrictions_interval_week_many'     => __dm('The discount can be used only %limit% in a %many% weeks.'),
                'detail_restrictions_interval_month_one'     => __dm('The discount can be used only %limit% a month.'),
                'detail_restrictions_interval_month_few'     => __dm('The discount can be used only %limit% in a %few% months.'),
                'detail_restrictions_interval_month_many'    => __dm('The discount can be used only %limit% in a %many% months.'),
                'detail_restrictions_interval_year_one'      => __dm('The discount can be used only %limit% a year.'),
                'detail_restrictions_interval_year_few'      => __dm('The discount can be used only %limit% in a %few% years.'),
                'detail_restrictions_interval_year_many'     => __dm('The discount can be used only %limit% in a %many% years.'),
                'detail_restrictions_interval_card_one'      => __dm('The discount can be used only %limit% per card.'),
                'detail_restrictions_from'                   => __dm('from'),
                'detail_restrictions_to'                     => __dm('to'),
                'detail_restrictions_age'                    => __dm('years of age'),
                'detail_restrictions_monday'                 => __dm('Monday'),
                'detail_restrictions_tuesday'                => __dm('Tuesday'),
                'detail_restrictions_wednesday'              => __dm('Wednesday'),
                'detail_restrictions_thursday'               => __dm('Thursday'),
                'detail_restrictions_friday'                 => __dm('Friday'),
                'detail_restrictions_saturday'               => __dm('Saturday'),
                'detail_restrictions_sunday'                 => __dm('Sunday'),
                'detail_restrictions_day'                    => __dm('days'),
                'detail_restrictions_week'                   => __dm('weeks'),
                'detail_restrictions_month'                  => __dm('months'),
                'detail_restrictions_year'                   => __dm('years'),
                'detail_restrictions_card'                   => __dm('card'),
                'infowindow_address'                         => __dm('Address'),
                'list_map_legend'                            => __dm('Legend'),
                'category_travel'                            => __dm('Travel'),
                'category_groceries'                         => __dm('Groceries'),
                'category_finance'                           => __dm('Finance'),
                'category_literature'                        => __dm('Literature'),
                'category_stationeryAndPrint'                => __dm('Stationery and print'),
                'category_pets'                              => __dm('Pets'),
                'category_photoAndVideo'                     => __dm('Photo and video'),
                'category_operators'                         => __dm('Operators'),
                'category_electronics'                       => __dm('Electronics'),
                'category_drugStore'                         => __dm('Drug store'),
                'category_gaming'                            => __dm('Gaming'),
                'category_flowers'                           => __dm('Flowers'),
                'category_fashion'                           => __dm('Fashion'),
                'category_healthAndBeauty'                   => __dm('Health and beauty'),
                'category_music'                             => __dm('Music'),
                'category_pharmacy'                          => __dm('Pharmacy'),
                'category_optics'                            => __dm('Optics'),
                'category_transportation'                    => __dm('Transportation'),
                'category_eshop'                             => __dm('E-shop'),
                'category_festival'                          => __dm('Festival'),
                'category_foodAndDrink'                      => __dm('Food and drink'),
                'category_shopping'                          => __dm('Shopping'),
                'category_culture'                           => __dm('Culture'),
                'category_skiResort'                         => __dm('Ski resort'),
                'category_sport'                             => __dm('Sport'),
                'category_accommodation'                     => __dm('Accommodation'),
                'category_education'                         => __dm('Education'),
                'category_funAndRelax'                       => __dm('Fun and relax'),
                'category_other'                             => __dm('Other'),
                'provider_online'                            => __dm('Online'),
                "sort_location_missing"                      => __dm('Your position has to be provided in order to sort by distance.'),
                'isic_discount'                              => __dm('ISIC discount'),
                'comment_share_your_experience'              => __dm('Share you experience'),
                'comment_customers_comments'                 => __dm('Customers\' comments'),
                'comment_username'                           => __dm('Name and surname'),
                'comment_cardnumber'                         => __dm('Card number'),
                'comment_your_comment'                       => __dm('Your comment'),
                'comment_send'                               => __dm('SEND COMMENT'),
                'comment_rating'                             => __dm('Discount rating'),
                'comment_no_so_far'                          => __dm('No comments yet, be the first one to share your experience with this discount.'),
                'username_cannot_be_empty'                   => __dm('Name and surname cannot be empty!'),
                'cardnumber_cannot_be_empty'                 => __dm('Card number cannot be empty!'),
                'text_cannot_be_empty'                       => __dm('Comment cannot be empty!'),
                'nothing_found'                              => __dm('No discounts were found.'),
                'try_something_else'                         => __dm('Try searching different query of view'),
                'all_discounts'                              => __dm('all discounts.'),
                'user_authentication_failed'                 => __dm('We could not verify your card. Please check your card number and name.'),
                'date_format'                                => __dm('Y-m-d H:i'),
                'moreBranches'                               => __dm("More branches"),
                'showProvider'                               => __dm("View discount"),
                'searchPlaceholderLong'                      => __dm("Enter expression to search (e.g. book Prague) and press enter..."),
                'searchPlaceholderShort'                     => __dm("Enter expression to search..."),
                'discounts_are_not_available_right_now'      => __dm("Discounts are not available right now.")
            )
        );

    }

    /**
     * Handle the shortcode logic.
     *
     * @param array $attributes Attributes passed to the shortcoce.
     */
    public function handler($attributes) {

        $attributes = $this->map_shortcode_attributes($attributes);
        $discount_manager_client = $this->get_dm_client();
        if (!is_null($discount_manager_client)) {
            $categories = $discount_manager_client->get_categories();
            if (isset($categories->items)) {
                $categories = $categories->items;
                $categoryMap = $this->compileCategories($categories);
                $data = [
                    'attributes' => $attributes,
                    'categories' =>	$categories,
                    'category_map' => $categoryMap,
                    'initial_content' => $this->get_initial_content(),
                    'ajax_url' => admin_url('admin-ajax.php')
                ];
            } else {
                $data = [
                    'attributes' => $attributes,
                    'categories' =>	[],
                    'category_map' => [],
                    'initial_content' => $this->get_initial_content(),
                    'ajax_url' => admin_url('admin-ajax.php'),
                    'error' => true,
                ];
            }
        } else {
            $data = [
                'attributes' => $attributes,
                'categories' =>	[],
                'category_map' => [],
                'initial_content' => $this->get_initial_content(),
                'ajax_url' => 'not initialized',
                'error' => true,
            ];
        }

        $template = new Template('discount-finder.phtml', $data);
        echo $template->build();
    }

    /**
     * Check that the current URL is cannonical URL of the provider.
     *
     * * Canonical URL is checked ony if the current URL matches pattern of the detail URL.
     * * Check that current URL matches the the cannoical URL of the provider with given identifier.
     * * Redirect user to canonical URL if necessary.
     *
     * FIXME: Does not handle non-existing providers.
     */
    private function checkCanonicalUrl() {
        foreach (array_keys($this->getProviderDetailUrlRewriteRules()) as $pattern) {
            // Check if current URL matches at least one of the URLs set in the administration.
            if (preg_match('#' . $pattern . '#', $_SERVER['REQUEST_URI'], $matches)) {
                $dmClient = $this->get_dm_client();
                if (!is_null($dmClient)) {
                    // Get provider with given identifier.
                    $provider = $dmClient->get_provider($matches[2]);
                    // Resolve provider canonical URL.
                    $providerCanonicalUrl = $this->get_provider_url($provider, $matches[1]);
                    if (strstr($providerCanonicalUrl, $_SERVER['REQUEST_URI']) === FALSE) {
                        // Redirect to cannonical URL if it does not match current URL.
                        wp_redirect($providerCanonicalUrl, 301);
                        exit;
                    }
                    break;
                }
            }
        }
    }

    /**
     * Resolve provider URL rewrite rulesbased on the URLs set in the administration interface.
     * Rewrite rules are returned as
     *
     * [
     *   {URL pattern to be matched} => {URL to which should it be rewritten}
     * ]
     *
     * @return array Array of the rewrite rule patterns.
     */
    private function getProviderDetailUrlRewriteRules() {
        $result = array();
        foreach (Config::used_in_pages() as $page) {
            $result["/?(.*$page)/[^/]+/([0-9]+)/?"] = 'index.php?pagename=' . $page . '&dmProviderId=$matches[2]';
        }
        return $result;
    }

    /**
     * Get initiali context that should be rendered for SEO purposes.
     */
    private function get_initial_content() {
        $context = $this->get_context();
        $data = [
            'context' => $context
        ];
        if ($context['providerId'] != null) {
            $provider = $this->get_active_context_provider();
            if ($provider == null) {
                return '';
            }
            $data ['provider'] = $provider;
            $template = new Template('discount-finder-provider-initial-content.phtml', $data);
            return $template->build();
        }
        $page = $context['crawlPage'] ? $context['crawlPage'] : 0;
        $searchParams = array(
                'offset'  => ($page - 1) * 50,
                'limit'   => 50,
                'sortKey' => 'createdOn'
        );
        $data['url'] = get_permalink();
        if (!empty($context['crawlPage']) && $context['crawlPage'] > 0) {
            if (!is_null($this->get_dm_client())) {
                $providers = $this->get_dm_client()->get_providers($searchParams);
                $data ['providers'] = [];
                foreach ($providers->items as $provider) {
                    $provider->providerUrl = $this->get_provider_url($provider);
                    $data['providers'][] = $provider;
                }
            }
        }
        $template = new Template('discount-finder-initial-content.phtml', $data);
        return $template->build();

    }

    /**
     * Get discount manager client. instance.
     * @return Discount_Manager_Client Discount manager client instance. Never null.
     */
    private function get_dm_client() {
        if ($this->dm_client == null) {
            if (empty(Config::url()) || empty(Config::username()) || empty(Config::password())) {
                $this->write_log('Error: Missing required DM client parameters');
            } else {
                $this->dm_client = new Discount_Manager_Client(Config::url(), Config::username(), Config::password(), Config::api_gateway_url(), Config::api_access_key_id(), Config::api_secret_access_key());
            }
        }
        return $this->dm_client;
    }

    /**
     * Map shortcode attributes, use defaults where not applicable.
     * @param array $attributes Atttributes of shortcode.
     * @return array Array of mapped attributes.
     */
    private function map_shortcode_attributes($attributes) {
        $attributes = shortcode_atts($this->default_shortcode_attributes, $attributes);
        // Resolve card types.
        if (!is_array($attributes['card_types'])) {
            $attributes['card_types'] = explode(',', $attributes['card_types']);
        }
        $cardTypes = array();
        foreach ($attributes['card_types'] as $cardType) {
            if (in_array($cardType, Config::supported_card_types())) {
                $cardTypes[] = strtolower(trim($cardType));
            }
        }
        //Resolve alternative name for card types
        if (!is_array($attributes['card_types_brands'])) {
            $brands = explode(',', $attributes['card_types_brands']);
            $processedBrands = array();
            foreach ($brands as $brand){
                if (!empty($brand)) {
                    $brand = explode("=", $brand);
                    $processedBrands[$brand[0]] = $brand[1];
                }
            }
            $attributes['card_types_brands'] = $processedBrands;
        }
        $attributes['card_types'] = $cardTypes;
        return $attributes;
    }

    /**
     * Get provider that is active within the current context.
     *
     * Cache the provider for this request.
     */
    private function get_active_context_provider() {
        $context = $this->get_context();
        if (empty($context['providerId'])) {
            return null;
        }
        if (($this->active_context_provider == null) && (!is_null($this->get_dm_client()))) {
            $this->active_context_provider = $this->get_dm_client()->get_provider($context['providerId']);
        }
        return $this->active_context_provider;
    }

    /**
     * @param $log
     */
    public function write_log ( $log )  {
        if ( true === WP_DEBUG ) {
            if ( is_array( $log ) || is_object( $log ) ) {
                error_log( print_r( $log, true ) );
            } else {
                error_log( $log );
            }
        }
    }
}
